#include "proteinMMEnzyme.h"

// #define DEBUG
// #define VERBOSE

using namespace std;

ProteinMMEnzyme::ProteinMMEnzyme(const std::string name, const double &timestep) :
Protein::Protein(name, timestep)
{
	function fu1;
	
	void (Protein::*pr1)(Protein *) = (void(Protein::*)(Protein *))(&ProteinMMEnzyme::reaction_A1);
	fu1.pfunction = pr1;
	setFunction("A1", fu1);

	void (Protein::*pr2)(Protein *) = (void(Protein::*)(Protein *))(&ProteinMMEnzyme::reaction_R1);
	fu1.pfunction = pr2;
	setFunction("R1", fu1);

	void (Protein::*pr3)(Protein *) = (void(Protein::*)(Protein *))(&ProteinMMEnzyme::reaction_D1);
	fu1.pfunction = pr3;
	setFunction("D1", fu1);
}


bool ProteinMMEnzyme::parseLine(char *linep)
{
	char  dummy[128]; // , pool0[128], pool1[128];
	int idx;

	// read data into the variables
	if(sscanf(linep, "%s %d %s %lf %lf %s %lf", dummy, &idx, SubstratePool, &kon_S, &k_conv, ProductPool, &koff_P) != 7) {
		cerr << "ProteinMMEnzyme::parseLine: not enough parameters for MMEnzyme line >>" << linep << "<<" << endl;
		return(false);
	}
	// Check signs of the rates
	if((kon_S < 0.0) || (k_conv < 0.0) || (koff_P < 0.0)) {
		cerr << "ProteinMMEnzyme::parseLine: negative rate constant in MMEnzyme line >>" << linep << "<<" << endl;
		return(false);
	}
	
	#ifdef VERBOSE
	cout << "------------------" << endl;
	cout << "   MMEnzyme: read: " << idx << endl;
	cout << "   MMEnzyme: read: " << SubstratePool << "  " << kon_S << endl;
	cout << "   MMEnzyme: read:              " << k_conv << endl;
	cout << "   MMEnzyme: read: " << ProductPool << "  " << koff_P << endl;
	#endif

	return(true);
}


void ProteinMMEnzyme::verify_pools()
{
	if(!(Pools->count(string(SubstratePool)))) {
		cerr << "ProteinMMEnzyme::verify_pools: pool "<<SubstratePool<<" not defined" << endl;
		exit(-3);
	}
	SubstratePoolp = getPool(SubstratePool);
	
	if(!(Pools->count(string(ProductPool)))) {
		cerr << "ProteinMMEnzyme::verify_pools: pool "<<ProductPool<<" not defined" << endl;
		exit(-3);
	}
	ProductPoolp = getPool(ProductPool);
}



// ++++++++++++++ internal reaction ++++++++++++++++++++++++

// A1: association of the substrate
void ProteinMMEnzyme::reaction_A1(Protein *p) {
	if(!bs_S && !bs_P) {
		if(SubstratePoolp->take_out(kon_S)) {
			bs_S = true;
			writeInternals();
		}
	}
}

// R1: conversion of substrate in bs_S into product in bs_P
void ProteinMMEnzyme::reaction_R1(Protein *p) {
	if(bs_S) 
	{
		switch(timer_R1) 
		{
			case 0: // initialize timer, start conversion
			{
				double t0 = 1.0 / (k_conv * delta_t);
				timer_R1 = static_cast<unsigned long int>(exp_ran(t0)); 
				
				bs_P = true;
				writeInternals();
			} break;
				
			case 1:   // conversion done
			{
				bs_S = false;
				timer_R1 = 0;
				writeInternals();
			} break;
				
			default:    // count down
				timer_R1--;
		}
	}
}


// D1: dissociation of the final product
void ProteinMMEnzyme::reaction_D1(Protein *p) 
{
	if(!bs_S && bs_P) 
	{
		switch(timer_D1) 
		{
			case 0:  // initialize timer
			{
				double t0 = 1.0 / (koff_P * delta_t);
				timer_D1 = static_cast<unsigned long int>(exp_ran(t0));
			} break;
				
			case 1:   // release!
			{
				bs_P = false;
				ProductPoolp->put_back();
				timer_D1 = 0;
				writeInternals();
			} break;
				
			default:    // count down
				timer_D1--;
		}
	}
}
		


std::string ProteinMMEnzyme::listInternals() {
	return(string("#time\tbs_S\tbs_P"));
}


void ProteinMMEnzyme::writeInternals()
{
	if(outputInternals)
	{
		double realTime = timer * delta_t;
																   
		internalsOutFile.open(internalFileName.c_str(), fstream::app);
		internalsOutFile << realTime << "\t" << bs_S << "\t" << bs_P << "\n";
		internalsOutFile.close();
	}
}
