#ifndef Pool_h
#define Pool_h

#include<string>
#include<stdlib.h>
#include<iostream>
#include<time.h>
#include<math.h>

#include"random.h"
// #include"parameter.h"


//! Implementation of Pools
 
/*!
This class implements the Pools of the model. Proteins can take out and put in molecules. 
A Pool can have an infinite number of particles. This is determined by a boolean value in the Constructor. 
If a pool is infinite one can take out or put in particles without changing the number of particles
keeping the concentration constant.
*/
class Pool
{
   public:
      //! Default constructor; creates an empty, infinite pool, set the time step
      Pool(const double &dt);
	  //! Constructor with initialization
	  Pool(long int num_of_particles_, double volume_, bool infinite_, std::string listed_, const double &dt);
      //! Destructor
      ~Pool();
      //! Copy construktor
      Pool(const Pool& aPool);

	  //! Make a pool infinite, i.e. have a constant concentration
	  void setInfinite(const bool &infinite_);

	  //! Check whether a pool should be listed in the output
	  bool isListed();
	  
	  //! @name Working with the pools
	  // @{
      //! Check whether the pool is empty or not
      bool is_empty() const;
      //! Get the concentration
      double getConcentration();
      //! Get the number of particles in the pool
      long int getInside() const  { return(num_of_particles); }
      //! Set the number of particles in this pool
      void setInside(const long int &num_of_particles_);
	  //! Return the volume of the pool
	  double getVolume() const { return(volume); }

	  //! Get content suitable for output: number or concentration depending on variable "listed"
	  double getOutput() const;
	  
      //! Take one particle out of this Pool with a given rate constant. 
      //! The pool determines the probability from the given rate and its internal concentration.
      bool take_out(const double &rate);
      //! Take "num" particles out of the pool.
      bool take(const long int &num);

      // Put a particle back into this pool with a given reaction rate.
      // The Pool calculates the corresponding probability from the rate alone 
      // (independent of its internal density). 
      // Note: this is inefficient, do not use
      // bool put_in(const double &rate);
	  //! Put a particle back into this Pool with P=1
	  void put_back();
	  //! Put N particles back into this Pool (unconditionally)
	  void put_back(const long int &N);
	  // @}

   private:
      //! Number of particles in this pool
      long int num_of_particles;
      //! Volume of the pool (can also be 2D, 1D, .., just be careful with the corresponding rate)
      double volume;
      //! Size of the timestep of the simulation
      double delta_t;
      //! Determines whether the Pool is infinite (= true)
      bool infinite;
	  //! True if pool is listed in the output
	  std::string listed;
};
#endif
