// Definitions for the automated inclusion into the parser
// LABEL:		DeltaPhi_ChemCap
// CLASSNAME:	ObservableDeltaPhiChemCapacitor
// 
#ifndef ObservableDeltaPhiChemCapacitor_h
#define ObservableDeltaPhiChemCapacitor_h

#include "observable.h"
#include "parameter.h"
#include <math.h>


//! Observable for the transmembrane potential of a spherical capacitor plus the chemical contribution.

//! Parameters are:
//! - r_i, the inner vesicle radius
//! - d, the membrane thickness
//! - epsilon, the relative eps of the membrane
//! - the voltage step per pH unit across the membrane
//!
//! The chemical contribution is calculated via the Nernst relation from
//! the pH difference between inside and outside of the vesicle.
//!
//! DeltaPhi accesses three pools: for the free protons inside the vesicle, 
//! for those bound to the membrane and for those created from internal charge 
//! separations in the proteins.
//!
//! The underlying physical model is a spherical capacitor with different 
//! dielectric constants. However, as only the voltage across the membrane 
//! is calculated, the epsilon outside of the membrane is not important, 
//! only the dielectric constant of the membrane itself.
//!
//! DeltaPhi is given in units of milliVolts [mV].

class ObservableDeltaPhiChemCapacitor : public Observable
{
	public:
	
	//! Constructor
	ObservableDeltaPhiChemCapacitor(const std::string name);
	//! Destructor
	~ObservableDeltaPhiChemCapacitor();
	
	//! Calculate and return the value of the transmembrane potential (in units of milliVolts [mV])
	double get_value();
	
	//! Parser for line from input file
	bool parseLine(char *linep);
	
	
	
	private:
	
	//! @name Char[]s for the pool names (see length above)
	//@{
	char HoutsidePool[MAX_POOLNAME_LENGTH], 
		 FreeHPool[MAX_POOLNAME_LENGTH], 
		 MembraneHPool[MAX_POOLNAME_LENGTH], 
		 ProteinHPool[MAX_POOLNAME_LENGTH],
		 c2redPool[MAX_POOLNAME_LENGTH],
		 c2oxPool[MAX_POOLNAME_LENGTH];
	//@}
	 
	//! name pointer to pools and variables for reaction constants to 
	//! avoid their continuous lookup
	//@{
	Pool *HoutsidePoolp, *FreeHPoolp, *MembraneHPoolp, *ProteinHPoolp, *c2redPoolp, *c2oxPoolp;
	double pHVoltage, r_a, d_M, r_i, eps, Q_offset;
	double freeHScaling, membraneHScaling, proteinHScaling, c2redScaling, c2oxScaling;
	//@}
	
	//! verify that all neccessary pools are defined
	void verify_pools();

};
#endif /* ObservableDeltaPhiChemCapacitor_h */
