// Definitions for the automated inclusion into the parser
// LABEL:		DeltaPhi_Chem
// CLASSNAME:	ObservableDeltaPhiNernst
// 
#ifndef ObservableDeltaPhiNernst_h
#define ObservableDeltaPhiNernst_h

#include "observable.h"
#include "parameter.h"
#include <math.h>



//! Observable for the transmembrane potential via the Nernst relation.

//! Subclass representing the transmembrane potential deltaPhi
//! via the Nernst relation of 59 mV per 1 pH unit.
//!
//! The one parameter is the potential per pH unit in mV
//!
//! DeltaPhi accesses two pools: for the free protons inside and outside 
//! of the vesicle.
//!
//! The Nernst relation relates the difference of the chemical potentials 
//! (i.e., the different densities of the charged protons on two sides of 
//! a membrane) to an equivalent transmembrane potential. It is
//! valid for infinite half-spaces on both sides of an infinite membrane.

class ObservableDeltaPhiNernst : public Observable
{
	public:
	
	//! Constructor
	ObservableDeltaPhiNernst(const std::string &name);
	//! Destructor
	~ObservableDeltaPhiNernst() { };

	//! Calculate and return the value of the transmembrane potential in units of milliVolts [mV]. 
	double get_value();
	
	//! Parser for input line
	bool parseLine(char *linep);
	
	
	private:
	
	//! @name Char[]s for the pool names (see length above)
	//@{
	char HinsidePool[MAX_POOLNAME_LENGTH], 
		 HoutsidePool[MAX_POOLNAME_LENGTH];
	//@}
	
	//! verify that all neccessary pools are defined
	void verify_pools();

	//! name pointer to pools and variables for reaction constants to 
	//! avoid their continuous lookup
	//@{
	Pool *HinsidePoolp, *HoutsidePoolp;
	double pHVoltage;
	//@}

};
#endif /* ObservableDeltaPhiNernst_h */
