#include "proteinRC.h"

// #define SCHWAETZ
// #define DEBUG

using namespace std;

ProteinRC::ProteinRC(const std::string name, const double &timestep) :
Protein::Protein(name, timestep)
{
	// initialize binding sites and internals
	// dark adapted state: SP with e-, Q bound at Qb, no c2
	bs_Q = true;
	bs_C = false;
	reg_SP = 1;
	reg_QHe = 0;
	reg_Qe = 0;
	
	function fu1;
	
	void (Protein::*pr1) (Protein *) = (void(Protein::*)(Protein *))(&ProteinRC::reaction1);
	fu1.pfunction = pr1;
	setFunction("r1", fu1);
	
	void (Protein::*pr2a) (Protein *) = (void(Protein::*)(Protein *))(&ProteinRC::reaction2a);
	fu1.pfunction = pr2a;
	setFunction("r2a", fu1);
	
	void (Protein::*pr2b) (Protein *) = (void(Protein::*)(Protein *))(&ProteinRC::reaction2b);
	fu1.pfunction = pr2b;
	setFunction("r2b", fu1);
	
	void (Protein::*pr3) (Protein *) = (void(Protein::*)(Protein *))(&ProteinRC::reaction3);
	fu1.pfunction = pr3;
	setFunction("r3", fu1);
	
	void (Protein::*pr4) (Protein *) = (void(Protein::*)(Protein *))(&ProteinRC::reaction4);
	fu1.pfunction = pr4;
	setFunction("r4", fu1);
	
	void (Protein::*pr5) (Protein *) = (void(Protein::*)(Protein *))(&ProteinRC::reaction5);
	fu1.pfunction = pr5;
	setFunction("r5", fu1);
	
	timer_r4 = 0;
	timer_r5 = 0;
}


std::string ProteinRC::listInternals() {
	return(string("#time\tbs_Q\tbs_C\treg_SP\treg_QHe\treg_Qe"));
}


void ProteinRC::writeInternals()
{
	if(outputInternals)
	{
		double realTime = timer * delta_t;
																   
		internalsOutFile.open(internalFileName.c_str(), fstream::app);
		internalsOutFile << realTime << "\t" << bs_Q << "\t" << bs_C << "\t" << reg_SP << "\t" << reg_QHe << "\t" << reg_Qe << "\n";
		internalsOutFile.close();
	}
}


void ProteinRC::verify_pools()
{
	if(!Pools->count(string(LHPool))) {
		cerr << "ProteinRC::verify_pools: pool "<< LHPool <<" not defined" << endl;
		exit(-3);
	}
	if(!Pools->count(string(QPool))) {
		cerr << "ProteinRC::verify_pools: pool "<< QPool <<" not defined" << endl;
		exit(-3);
	}
	if(!Pools->count(string(QH2Pool))) {
		cerr << "ProteinRC::verify_pools: pool "<< QH2Pool <<" not defined" << endl;
		exit(-3);
	}
	if(!Pools->count(string(c2redPool))) {
		cerr << "ProteinRC::verify_pools: pool "<< c2redPool <<" not defined" << endl;
		exit(-3);
	}
	if(!Pools->count(string(c2oxPool))) {
		cerr << "ProteinRC::verify_pools: pool "<< c2oxPool <<" not defined" << endl;
		exit(-3);
	}
	if(!Pools->count(string(HoutsidePool))) {
		cerr << "ProteinRC::verify_pools: pool "<< HoutsidePool <<" not defined" << endl;
		exit(-3);
	}
	if(!Pools->count(string(HInternalPool))) {
		cerr << "ProteinRC::verify_pools: pool "<< HInternalPool <<" not defined" << endl;
		exit(-3);
	}
	
	LHPoolp        = getPool(LHPool);
	QPoolp         = getPool(QPool);
	QH2Poolp       = getPool(QH2Pool);
	c2redPoolp     = getPool(c2redPool);
	c2oxPoolp      = getPool(c2oxPool);
	HoutsidePoolp  = getPool(HoutsidePool);
	HInternalPoolp = getPool(HInternalPool);
}

bool ProteinRC::parseLine(char *linep)
{
	char  dummy[128];
	int num_of_proteins;

	// read date into the variables
	if(sscanf(linep, "%s %d %s %lf %s %lf %s %lf %s %lf %s %lf %s %lf %s", 
		dummy, &num_of_proteins, LHPool, &LH_kon, HoutsidePool, &Houtside_kon, QPool, &Q_kon, QH2Pool, &QH2_koff, c2redPool, &c2red_kon, c2oxPool, &c2ox_koff, HInternalPool) != 15) {
		cerr << "proteinRC::parseLine: not enough parameters for RC line >>" << linep << "<<" << endl;
		return(false);
	}
	// Check signs of the rates
	if((LH_kon < 0.0) || (Houtside_kon < 0.0) || (Q_kon < 0.0) || (QH2_koff < 0.0) || (c2red_kon < 0.0) || (c2ox_koff < 0.0)) {
		cerr << "proteinRC::parseLine: negative rate constant in RC line >>" << linep << "<<" << endl;
		return(false);
	}

	#ifdef SCHWAETZ
	cout << "------------------" << endl;
	cout << "   RC: read: " << num_of_proteins << endl;
	cout << "   RC: read: " << LHPool << "  " << LH_kon << endl;
	cout << "   RC: read: " << HoutsidePool << "  " << Houtside_kon << endl;
	cout << "   RC: read: " << QPool << "  " << Q_kon << endl;
	cout << "   RC: read: " << QH2Pool << "  " << QH2_koff << endl;
	cout << "   RC: read: " << c2redPool << "  " << c2red_kon << endl;
	cout << "   RC: read: " << c2oxPool << "  " << c2ox_koff << endl;
	cout << "   RC: read: " << HInternalPool << endl;
	#endif

	return(true);
}


long int ProteinRC::get_c2ox()
{
	// c2 bound and SP reduced -> c2 could only bind if SP was oxidized before
	if(bs_C && (reg_SP == 1)) {
		return(1L);
	}
	return(0L);
}


// ++++++++++++++ internal reactions ++++++++++++++++++++++++

// Bind a Q to the RC
void ProteinRC::reaction1(Protein *p)
{
	if (!bs_Q)
	{
		if (QPoolp->take_out(Q_kon))
		{
			bs_Q = true;
			writeInternals();
			#ifdef DEBUG
			cout << "ProteinRC::reaction1: is done\n";
			#endif
		}
		#ifdef DEBUG
		else {
			cout << "ProteinRC::reaction1: no Q availabe\n";
		}
		#endif
	}
	#ifdef VERBOSE
	else {
		cout << "impossible: Q is bound\n";
	}
	#endif
}

   
// Transfer  an electron to the quinone.
// Due to the charge separation a (virtual) charge is put into the HInternal pool
void ProteinRC::reaction2a(Protein *p)
{
	if (bs_Q && (reg_SP == 1) && (reg_Qe == 0) && ((reg_QHe == 0) || (reg_QHe == 1)))
	{
		if (LHPoolp->take_out(LH_kon))
		{
			reg_SP = 0;
			reg_Qe = 1;
			HInternalPoolp->put_back();
			writeInternals();
			#ifdef DEBUG
			cout << "ProteinRC::reaction2a: done, 1 charge into HInternalpool\n";
			#endif
		}
// 		#ifdef DEBUG
// 		else {
// 			cout << "ProteinRC::reaction2: no photon availabe\n";
// 		}
// 		#endif
	}
	#ifdef VERBOSE
	else {
		cout << "impossible: not in state 2\n";
	}
	#endif
}


// Transfer a proton to the quinone
void ProteinRC::reaction2b(Protein *p)
{
	if (bs_Q && (reg_Qe == 1) && ((reg_QHe == 0) || (reg_QHe == 1)))
	{
		if (HoutsidePoolp->take_out(Houtside_kon))
		{
			reg_QHe++;
			reg_Qe = 0;
			writeInternals();
			#ifdef DEBUG
			cout << "ProteinRC::reaction2b: done\n";
			#endif
		}
// 		#ifdef DEBUG
// 		else {
// 			cout << "ProteinRC::reaction2: no proton availabe\n";
// 		}
// 		#endif
	}
	#ifdef VERBOSE
	else {
		cout << "impossible: not in state 2\n";
	}
	#endif
}


// Bind a Cytochrome c2
void ProteinRC::reaction3(Protein *p)
{
	if ((!bs_C) && (reg_SP == 0))
	{
		if (c2redPoolp->take_out(c2red_kon))
		{
			bs_C = true;
			writeInternals();
			#ifdef DEBUG
			cout << "ProteinRC::reaction3: is processing\n";
			#endif
		}
// 		#ifdef DEBUG
// 		else {
// 			cout << "ProteinRC::reaction3: no c2red available\n";
// 		}
// 		#endif
	}
	#ifdef VERBOSE
	else {
		cout << "impossible: not in state 3\n";
	}
	#endif
}


// Transfer an electron from c2 to special pair and unbind c2,
// take back the virtual proton from the HInternal pool.
// Does e- transfer before the c2 unbinding starts
void ProteinRC::reaction4(Protein *p)
{
	if(bs_C)
	{
		switch(timer_r4)
		{
			case 0:
			{
				// initialize timer
				double t0 = 1.0/(c2ox_koff*delta_t);
				timer_r4 = static_cast<unsigned long int>(exp_ran(t0));
				if(timer_r4 == 0) { timer_r4 = 1; }
				
				// charge transfer c2 -> P
				HInternalPoolp->take(1L);
				reg_SP = 1;
				writeInternals();
				
				#ifdef DEBUG
				cout << "ProteinRC::reaction4: is processing, took one charge from HInternalPools,  timer_r4 = " << timer_r4 << endl;
				#endif
			}; break;
			case 1:
			{
				c2oxPoolp->put_back();
				bs_C = false;
				writeInternals();
				timer_r4 = 0;
				
				#ifdef DEBUG
				cout << "ProteinRC::reaction4: is done\n";
				#endif
			}; break;
			default: 
				timer_r4--;
		}
	}
	#ifdef VERBOSE
	else
		{
		cout << "impossible: not in state 4\n";
	}
	#endif
}


// Unbind the QH2
void ProteinRC::reaction5(Protein *p)
{
	if (bs_Q && (reg_QHe == 2))
	{
		switch(timer_r5)
		{
			case 0:
			{
				double t0 = 1.0/(QH2_koff*delta_t);
				timer_r5 = static_cast<unsigned long int>(exp_ran(t0));
				if(timer_r5 == 0) { timer_r5 = 1; }

				#ifdef DEBUG
				cout << "ProteinRC::reaction5 is processing: timer_r5 = " << timer_r5 << endl;
				#endif
			}; break;
			case 1:
			{
				QH2Poolp->put_back();
				bs_Q = false;
				reg_QHe = 0;
				writeInternals();
				timer_r5 = 0;
				#ifdef DEBUG
				cout << "ProteinRC::reaction5: is done\n";
				#endif
			}; break;
			default: 
				timer_r5--;
		}
	}
	#ifdef VERBOSE
	else
		{
		cout << "impossible: not in state 5\n";
	}
	#endif
}
